@@ -24,25 +24,22 @@
 
 #include <sys/time.h>
 #include <time.h>
+#include <string.h>
+#include <limits.h>
 
 #include <glib/glist.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktreemodel.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <gtk/gtk.h>
 
 #include "sj-error.h"
 #include "sj-extracting.h"
 #include "sj-util.h"
 #include "sj-play.h"
+#include "sj-inhibit.h"
+#include "sj-genres.h"
 
 typedef struct {
   int seconds;
@@ -51,6 +48,26 @@ typedef struct {
   int taken;
 } Progress;
 
+
+typedef enum {
+  OVERWRITE_ALL = 1,
+  SKIP_ALL = 2,
+  NORMAL = 3
+} OverWriteModes;
+
+int overwrite_mode;
+
+typedef enum {
+  BUTTON_SKIP = 1,
+  BUTTON_SKIP_ALL = 2,
+  BUTTON_OVERWRITE = 3,
+  BUTTON_OVERWRITE_ALL = 4,
+} OverwriteDialogResponse;
+
+/* files smaller than this are assumed to be corrupt */
+#define MIN_FILE_SIZE 100000
+
+
 /** If this module has been initialised yet. */
 static gboolean initialised = FALSE;
 
@@ -58,19 +75,12 @@ static gboolean initialised = FALSE;
 static GtkWidget *progress_bar, *status_bar;
 
 /** The widgets in the main UI */
-static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *genre_entry, *track_listview;
+static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *genre_entry, *year_entry, *disc_number_entry, *track_listview;
 
 /** The menuitem in the main menu */
 static GtkWidget *extract_menuitem, *play_menuitem, *reread_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 
-/** The current TrackDetails being extracted. */
-static TrackDetails *track;
-
-/**
- * The list of pending TrackDetails to extract. ->data points to existing
- * entries, so do not free the data, only the list.
- */
-static GList *pending;
+static GtkTreeIter current;
 
 /**
  * A list of paths we have extracted music into. Contains allocated items, free
@@ -99,18 +109,26 @@ static int total_duration;
  */
 static Progress before;
 
+/**
+* The cookie returned from PowerManagement
+*/
+static guint cookie;
+
 /**
  * Build the absolute filename for the specified track.
  *
- * The base path is the extern variable 'base_path', the format to use
- * is the extern variable 'file_pattern'. Free the result when you
+ * The base path is the extern variable 'base_uri', the formats to use are the
+ * extern variables 'path_pattern' and 'file_pattern'. Free the result when you
  * have finished with it.
  */
 static char*
-build_filename (const TrackDetails *track)
+build_filename (const TrackDetails *track, gboolean temp_filename, GError **error)
 {
-  GnomeVFSURI *uri, *new;
-  char *realfile, *realpath, *filename, *string;
+  GnomeVFSURI *uri, *new; 
+  gchar *realfile, *realpath, *filename, *string;
+  const gchar *extension;
+  size_t len_extension;
+  int max_realfile = INT_MAX;
   GMAudioProfile *profile;
 
   g_object_get (extractor, "profile", &profile, NULL);
@@ -122,8 +140,27 @@ build_filename (const TrackDetails *track)
   gnome_vfs_uri_unref (uri); uri = new;
   g_free (realpath);
 
+  extension = gm_audio_profile_get_extension (profile);
+  len_extension = 1 + strlen (extension);
+#if defined(NAME_MAX) && NAME_MAX > 0
+  max_realfile = NAME_MAX - len_extension;
+#endif /* NAME_MAX */
+#if defined(PATH_MAX) && PATH_MAX > 0
+  if (!strcmp (gnome_vfs_uri_get_scheme (uri), "file")) {
+    size_t len_path = strlen (gnome_vfs_uri_get_path (uri)) + 1;
+    max_realfile = MIN (max_realfile, PATH_MAX - len_path - len_extension);
+  }
+#endif /* PATH_MAX */
+  if (max_realfile <= 0) {
+    g_set_error (error, SJ_ERROR, SJ_ERROR_INTERNAL_ERROR, g_strdup (gnome_vfs_result_to_string (GNOME_VFS_ERROR_NAME_TOO_LONG)));
+    return NULL;
+  }
   realfile = filepath_parse_pattern (file_pattern, track);
-  filename = g_strdup_printf("%s.%s", realfile, gm_audio_profile_get_extension(profile));
+  if (temp_filename) {
+    filename = g_strdup_printf (".%.*s.%s", max_realfile-1, realfile, extension);
+  } else {
+    filename = g_strdup_printf ("%.*s.%s", max_realfile, realfile, extension);
+  }
   new = gnome_vfs_uri_append_file_name (uri, filename);
   gnome_vfs_uri_unref (uri); uri = new;
   g_free (filename); g_free (realfile);
@@ -133,6 +170,21 @@ build_filename (const TrackDetails *track)
   return string;
 }
 
+static gboolean
+find_next (void)
+{
+  do {
+    gboolean extract = FALSE;
+    
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &current, COLUMN_EXTRACT, &extract, -1);
+    
+    if (extract)
+      return TRUE;
+    
+  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (track_store), &current));
+  return FALSE;
+}
+
 /**
  * Cleanup the data used, and even enable the Extract button again.
  */
@@ -142,18 +194,24 @@ cleanup (void)
   /* We're not extracting any more */
   extracting = FALSE;
 
+  nautilus_burn_drive_unlock (drive);
+
+  sj_uninhibit (cookie);
+
   /* Remove any state icons from the model */
-  gtk_list_store_set (track_store, &track->iter,
-                 COLUMN_STATE, STATE_IDLE, -1);
-  
+  if (current.stamp) {
+    /* TODO: has to be a better way to do that test */
+    gtk_list_store_set (track_store, &current,
+                        COLUMN_STATE, STATE_IDLE, -1);
+  }
+
   /* Free the used data */
   if (paths) {
     g_list_deep_free (paths, NULL);
     paths = NULL;
   }
-  g_list_free (pending);
-  pending = NULL;
-  track = NULL;
+  /* Forcibly invalidate the iterator */
+  current.stamp = 0;
   /* TODO: find out why GTK+ needs this to work (see #364371) */
   gtk_button_set_label (GTK_BUTTON (extract_button), _("Extract"));
   gtk_button_set_label (GTK_BUTTON (extract_button), SJ_STOCK_EXTRACT);
@@ -162,11 +220,14 @@ cleanup (void)
   gtk_statusbar_push (GTK_STATUSBAR (status_bar), 0, "");
   /* Clear the progress bar */
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0);
+  gtk_widget_hide (progress_bar);
   
   gtk_widget_set_sensitive (play_button, TRUE);
   gtk_widget_set_sensitive (title_entry, TRUE);
   gtk_widget_set_sensitive (artist_entry, TRUE);
   gtk_widget_set_sensitive (genre_entry, TRUE);
+  gtk_widget_set_sensitive (year_entry, TRUE);
+  gtk_widget_set_sensitive (disc_number_entry, TRUE);
   /* Enabling the Menuitem */ 
   gtk_widget_set_sensitive (play_menuitem, TRUE);
   gtk_widget_set_sensitive (extract_menuitem, TRUE);
@@ -181,35 +242,44 @@ cleanup (void)
   g_signal_handlers_unblock_by_func (track_listview, on_tracklist_row_activate, NULL);
 }
 
+
 /**
  * Check if a file exists, can be written to, etc.
  * Return true on continue, false on skip.
  */
-static gboolean
-check_for_file (const char* uri)
+static GnomeVFSFileSize
+check_file_size (const char* uri)
 {
   GnomeVFSFileInfo info;
   GnomeVFSResult res;
-  GtkWidget *dialog;
-  char *filename, *size;
-  int ret;
-  
+ 
   res = gnome_vfs_get_file_info (uri, &info, GNOME_VFS_FILE_INFO_DEFAULT);
+
+  /* No existing file */
   if (res == GNOME_VFS_ERROR_NOT_FOUND)
-    return TRUE;
+    return 0;
+
+  /* unexpected error condition - bad news */
   if (res != GNOME_VFS_OK) {
     /* TODO: display an error dialog */
     g_warning ("Cannot get file info: %s", gnome_vfs_result_to_string (res));
-    return FALSE;
+    return -1;
   }
-  if (info.size < (100000)) {
-    /* The file exists but is small, assume overwriting */
-    return TRUE;
-  }
-  /* Otherwise the file exists and is large, ask user if they really
-     want to overwrite it */
+
+  /* A file with that name does exist. Report the size. */
+  return info.size;
+}
+
+
+static gboolean
+confirm_overwrite_existing_file (const char* uri, int *overwrite_mode, GnomeVFSFileSize info_size)
+{ 
+  OverwriteDialogResponse ret;
+  GtkWidget *dialog;
+  char *filename, *size;
+ 
   filename = gnome_vfs_format_uri_for_display (uri);
-  size = gnome_vfs_format_file_size_for_display (info.size);
+  size = gnome_vfs_format_file_size_for_display (info_size);
   dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), GTK_DIALOG_MODAL,
                                    GTK_MESSAGE_QUESTION,
                                    GTK_BUTTONS_NONE,
@@ -217,15 +287,40 @@ check_for_file (const char* uri)
                                    filename, size);
   g_free (filename);
   g_free (size);
-  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Skip"), GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Overwrite"), GTK_RESPONSE_ACCEPT);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Skip"), BUTTON_SKIP);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("S_kip All"), BUTTON_SKIP_ALL);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Overwrite"), BUTTON_OVERWRITE);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("Overwrite _All"), BUTTON_OVERWRITE_ALL);
+
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), BUTTON_SKIP);
   gtk_widget_show_all (dialog);
   ret = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
-  return ret == GTK_RESPONSE_ACCEPT;
+
+  switch (ret) {
+    case BUTTON_OVERWRITE_ALL:
+      *overwrite_mode = OVERWRITE_ALL;
+      return TRUE;
+      break;
+    case BUTTON_OVERWRITE:
+      return TRUE;
+      break;
+    case BUTTON_SKIP_ALL:
+      *overwrite_mode = SKIP_ALL;
+      return FALSE;
+      break;
+    case BUTTON_SKIP:
+    case GTK_RESPONSE_DELETE_EVENT:
+    default:
+      return FALSE;
+      break;
+  }
+
+
+return ret;
 }
 
+
 /**
  * Find the name of the directory this file is in, create it, and return the
  * directory.
@@ -245,7 +340,9 @@ create_directory_for (const char* url, GError **error)
 
   res = make_directory_with_parents_for_uri (parent, 0777);
   if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_FILE_EXISTS) {
-    g_set_error (error, SJ_ERROR, SJ_ERROR_CD_PERMISSION_ERROR, g_strdup (gnome_vfs_result_to_string (res)));
+    g_set_error (error, SJ_ERROR, SJ_ERROR_CD_PERMISSION_ERROR,
+                 _("Failed to create output directory: %s"),
+                 gnome_vfs_result_to_string (res));
     return NULL;
   }
 
@@ -254,82 +351,92 @@ create_directory_for (const char* url, GError **error)
   return string;
 }
 
-/* Prototype for pop_and_extract */
+/* Prototypes for pop_and_extract */
 static void on_completion_cb (SjExtractor *extractor, gpointer data);
+static void on_error_cb (SjExtractor *extractor, GError *error, gpointer data);
 
 /**
  * The work horse of this file.  Take the first entry from the pending list,
  * update the UI, and start the extractor.
  */
 static void
-pop_and_extract (void)
+pop_and_extract (int *overwrite_mode)
 {
-  if (pending == NULL) {
+  if (current.stamp == 0) {
+    /* TODO: remove this test? */
     g_assert_not_reached ();
   } else {
-    char *file_path, *directory;
+    TrackDetails *track = NULL;
+    char *file_path = NULL, *temp_file_path = NULL, *directory;
     GError *error = NULL;
-    char *text;
-    
+
     /* Pop the next track to extract */
-    track = pending->data;
-    pending = g_list_remove (pending, track);
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &current, COLUMN_DETAILS, &track, -1);
     /* Build the filename for this track */
-    file_path = build_filename (track);
+    file_path = build_filename (track, FALSE, &error);
+    if (error) {
+      goto error;
+    }
+    temp_file_path = build_filename (track, TRUE, &error);
+    if (error) {
+      goto error;
+    }
     /* And create the directory it lives in */
     directory = create_directory_for (file_path, &error);
     if (error) {
-      GtkWidget *dialog;
-      text = g_strdup_printf (_("Sound Juicer could not extract this CD.\nReason: %s"), error->message);
-
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), 0,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       text);
-      g_free (text);
-
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      g_error_free (error);
-      cleanup ();
-      return;
+      goto error;
     }
     /* Save the directory name for later */
     paths = g_list_append (paths, directory);
-    /* See if the file exists */
-    if (!check_for_file (file_path)) {
-      on_completion_cb (NULL, NULL);
+
+
+    GnomeVFSFileSize file_size;
+    file_size = check_file_size (file_path);
+
+    /* Skip if destination file can't be accessed (unexpected error). */
+    /* Skip existing files if "skip all" is selected. */
+    if ((file_size == -1) ||
+        ((file_size > MIN_FILE_SIZE) && (*overwrite_mode == SKIP_ALL))) {
+      on_completion_cb (NULL, overwrite_mode);
       return;
     }
-   
+
+    /* What if the file already exists? */
+    if ((file_size > MIN_FILE_SIZE) &&
+        (*overwrite_mode != OVERWRITE_ALL) &&
+        (confirm_overwrite_existing_file (file_path, overwrite_mode, file_size) == FALSE)) {
+      on_completion_cb (NULL, overwrite_mode);
+      return;
+    }
+
+    /* OK, we can write/overwrite the file */
+
+
     /* Update the state stock image */
-    gtk_list_store_set (track_store, &track->iter,
+    gtk_list_store_set (track_store, &current,
                    COLUMN_STATE, STATE_EXTRACTING, -1);
-		
+    
     /* Update the progress bars */
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar),
                                    CLAMP ((float)current_duration / (float)total_duration, 0.0, 1.0));
 
+    /* Set the Treelist focus to the item to be extracted */
+    GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL (track_store), &current); 
+    gtk_tree_view_set_cursor(GTK_TREE_VIEW (track_listview), path, NULL, TRUE);
+    gtk_tree_path_free(path);
+
     /* Now actually do the extraction */
-    sj_extractor_extract_track (extractor, track, file_path, &error);
+    sj_extractor_extract_track (extractor, track, temp_file_path, &error);
     if (error) {
-      GtkWidget *dialog;
-
-      text = g_strdup_printf (_("Sound Juicer could not extract this CD.\nReason: %s"), error->message);
-
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), 0,
-                                       GTK_MESSAGE_ERROR,
-                                       GTK_BUTTONS_CLOSE,
-                                       text);
-      g_free (text);
-
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
-      g_error_free (error);
-      cleanup ();
-      return;
+      goto error;
     }
+    goto local_cleanup;
+error:
+    on_error_cb (NULL, error, NULL);
+    g_error_free (error);
+local_cleanup:
     g_free (file_path);
+    g_free (temp_file_path);
   }
 }
 
@@ -348,8 +455,6 @@ extract_track_foreach_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *i
                       COLUMN_DETAILS, &track,
                       -1);
   if (extract) {
-    track->iter = *iter;
-    pending = g_list_append (pending, track);
     ++total_extracting;
     total_duration += track->duration;
   }
@@ -365,7 +470,7 @@ update_speed_progress (SjExtractor *extractor, float speed, int eta)
   char *eta_str;
 
   if (eta >= 0) {
-    eta_str = g_strdup_printf (_("Estimated time left: %d:%02d (at %0.1f\303\227)"), eta / 60, eta % 60, speed);	
+    eta_str = g_strdup_printf (_("Estimated time left: %d:%02d (at %0.1f\303\227)"), eta / 60, eta % 60, speed);  
   } else {
     eta_str = g_strdup (_("Estimated time left: unknown"));  
   }
@@ -396,10 +501,10 @@ on_progress_cb (SjExtractor *extractor, const int seconds, gpointer data)
 
       gettimeofday(&time, NULL);
       taken = time.tv_sec + (time.tv_usec / 1000000.0)
-        - (before.time.tv_sec + (before.time.tv_usec / 1000000.0));	
+        - (before.time.tv_sec + (before.time.tv_usec / 1000000.0)); 
       if (taken >= 4) {
-	before.taken += taken;
-	before.ripped += current_duration + seconds - before.seconds;
+        before.taken += taken;
+        before.ripped += current_duration + seconds - before.seconds;
         speed = (float) before.ripped / (float) before.taken;
         update_speed_progress (extractor, speed, (int) ((total_duration - current_duration - seconds) / speed));
         before.seconds = current_duration + seconds;
@@ -444,63 +549,40 @@ base_finder (char *path, char **ret)
 static gboolean
 on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 {
-	gtk_window_set_urgency_hint (GTK_WINDOW (main_window), FALSE);
-	return FALSE;
+  gtk_window_set_urgency_hint (GTK_WINDOW (main_window), FALSE);
+  return FALSE;
 }
 
 /**
- * Show the "Finished!" dialog, allowing the user to open Nautilus if he wants.
+ * Handle any post-rip actions
  */
 static void
-show_finished_dialog (const TrackDetails *track)
+finished_actions (void)
 {
-  GtkWidget *dialog;
-  int result;
-  char *base;
-
-  base = NULL;
-  /* Find the deepest common directory. */
-  g_list_foreach (paths, (GFunc)base_finder, &base);
-
-  cleanup ();
-
-  char *message;
-  message = g_strdup_printf (_("%s has been copied successfully."), track->album->title);
-
-  dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-                                   GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_INFO,
-                                   GTK_BUTTONS_NONE,
-                                   (message));
-
-  g_free (message);
-  /* If we eject when finished, eject now, otherwise add a button */
-  if (eject_finished) {
-    nautilus_burn_drive_eject (drive);
-  } else {
-    gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Eject"), 2, NULL);
-  }
-
   /* Trigger glowing effect after copy */
-  g_signal_connect (G_OBJECT (dialog), "focus-in-event",  G_CALLBACK (on_main_window_focus_in),  NULL);
+  g_signal_connect (G_OBJECT (main_window), "focus-in-event",
+                    G_CALLBACK (on_main_window_focus_in),  NULL);
   gtk_window_set_urgency_hint (GTK_WINDOW (main_window), TRUE);
 
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          GTK_STOCK_OPEN, 1,
-                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                          NULL);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
-  result = gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
-  if (result == 1) {
-    char *command;
-    command = g_strdup_printf ("gnome-open \"%s\"", base);
-    g_spawn_command_line_async (command, NULL);
-    g_free (command);
-  } else if (result == 2) {
+  /* Maybe eject */
+  if (eject_finished) {
     nautilus_burn_drive_eject (drive);
   }
-  g_free (base);
+  
+  /* Maybe open the target directory */
+  if (open_finished) {
+    char *base, *command;
+
+    base = NULL;
+    /* Find the deepest common directory. */
+    g_list_foreach (paths, (GFunc)base_finder, &base);
+    
+    command = g_strdup_printf ("gnome-open \"%s\"", base);
+    g_spawn_command_line_async (command, NULL);
+    
+    g_free (command);
+    g_free (base);
+  }
 }
 
 /**
@@ -509,25 +591,45 @@ show_finished_dialog (const TrackDetails *track)
 static void
 on_completion_cb (SjExtractor *extractor, gpointer data)
 {
+  TrackDetails *track = NULL;
+  char *temp_file_path, *new_file_path;
+
   /* Only manipulate the track state if we have an album, as we might be here if
      the disk was ejected mid-rip. */
   if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (track_store), NULL) > 0) {
     /* Remove the track state */
-    gtk_list_store_set (track_store, &track->iter, COLUMN_STATE, STATE_IDLE, -1);
+    gtk_list_store_set (track_store, &current, COLUMN_STATE, STATE_IDLE, -1);
     /* Uncheck the Extract check box */
-    gtk_list_store_set (track_store, &track->iter, COLUMN_EXTRACT, FALSE, -1);
+    gtk_list_store_set (track_store, &current, COLUMN_EXTRACT, FALSE, -1);
   }
   
-  if (pending == NULL) {
-    show_finished_dialog (track);
-    if (autostart) {
-      gtk_main_quit ();
-    }
-  } else {
+  gtk_tree_model_get (GTK_TREE_MODEL (track_store), &current,
+                      COLUMN_DETAILS, &track, -1);
+
+
+
+    temp_file_path = build_filename (track, TRUE, NULL);
+    new_file_path = build_filename (track, FALSE, NULL);
+    gnome_vfs_move (temp_file_path, new_file_path, TRUE);
+
+    g_free (temp_file_path);
+    g_free (new_file_path);
+
+  if (find_next ()) {
     /* Increment the duration */
     current_duration += track->duration;
     /* And go and do it all again */
-    pop_and_extract ();
+    pop_and_extract ((int*)data);
+  } else {
+    /* If we got here then the track state has been set to IDLE already, so
+       unset the current iterator */
+    current.stamp = 0;
+    finished_actions ();
+    cleanup ();
+
+    if (autostart) {
+      gtk_main_quit ();
+    }
   }
 }
 
@@ -561,7 +663,18 @@ on_error_cb (SjExtractor *extractor, GError *error, gpointer data)
 void
 on_progress_cancel_clicked (GtkWidget *button, gpointer user_data)
 {
+  TrackDetails *track = NULL;
+  char *file_path;
+
   sj_extractor_cancel_extract (extractor);
+  
+  gtk_tree_model_get (GTK_TREE_MODEL (track_store), &current,
+                      COLUMN_DETAILS, &track, -1);
+
+  file_path = build_filename (track, TRUE, NULL);
+  gnome_vfs_unlink (file_path);
+  g_free (file_path);
+
   cleanup ();
 }
 
@@ -571,6 +684,8 @@ on_progress_cancel_clicked (GtkWidget *button, gpointer user_data)
 void
 on_extract_activate (GtkWidget *button, gpointer user_data)
 {
+  char *reason;
+  
   /* first make sure we're not playing, we cannot share the resource */
   stop_playback ();
   
@@ -579,15 +694,16 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
      on_progress_cancel_clicked (NULL, NULL);
      return;
   }
-				
+        
   /* Populate the pending list */
-  pending = NULL;
+  current.stamp = 0;
   total_extracting = 0;
   current_duration = total_duration = 0;
   before.seconds = -1;
+  overwrite_mode = NORMAL;
   gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), extract_track_foreach_cb, NULL);
   /* If the pending list is still empty, return */
-  if (pending == NULL) {
+  if (total_extracting == 0) {
     /* Should never reach here */
     g_warning ("No tracks selected for extracting");
     return;
@@ -597,7 +713,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   if (!initialised) {
     /* Connect to the SjExtractor signals */
     g_signal_connect (extractor, "progress", G_CALLBACK (on_progress_cb), NULL);
-    g_signal_connect (extractor, "completion", G_CALLBACK (on_completion_cb), NULL);
+    g_signal_connect (extractor, "completion", G_CALLBACK (on_completion_cb), (gpointer)&overwrite_mode);
     g_signal_connect (extractor, "error", G_CALLBACK (on_error_cb), NULL);
 
     extract_button = glade_xml_get_widget (glade, "extract_button");
@@ -605,10 +721,12 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     title_entry = glade_xml_get_widget (glade, "title_entry");
     artist_entry = glade_xml_get_widget (glade, "artist_entry");
     genre_entry = glade_xml_get_widget (glade, "genre_entry");
+    year_entry = glade_xml_get_widget (glade, "year_entry");
+    disc_number_entry = glade_xml_get_widget (glade, "disc_number_entry");
     track_listview = glade_xml_get_widget (glade, "track_listview");
     progress_bar = glade_xml_get_widget (glade, "progress_bar");
     status_bar = glade_xml_get_widget (glade, "status_bar");
-	
+  
     play_menuitem = glade_xml_get_widget (glade, "play_menuitem");
     extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
     reread_menuitem = glade_xml_get_widget (glade, "re-read");
@@ -622,6 +740,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   /* TODO: find out why GTK+ needs this to work (see #364371) */
   gtk_button_set_label (GTK_BUTTON (extract_button), _("Stop"));
   gtk_button_set_label (GTK_BUTTON (extract_button), GTK_STOCK_STOP);
+  gtk_widget_show (progress_bar);
   
   /* Reset the progress dialog */
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0);
@@ -632,8 +751,10 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   gtk_widget_set_sensitive (title_entry, FALSE);
   gtk_widget_set_sensitive (artist_entry, FALSE);
   gtk_widget_set_sensitive (genre_entry, FALSE);
+  gtk_widget_set_sensitive (year_entry, FALSE);
+  gtk_widget_set_sensitive (disc_number_entry, FALSE);
 
-  /* Disable the menuitems in the main menu*/	
+  /* Disable the menuitems in the main menu*/ 
   gtk_widget_set_sensitive (play_menuitem, FALSE);
   gtk_widget_set_sensitive (extract_menuitem, FALSE);
   gtk_widget_set_sensitive (reread_menuitem, FALSE);
@@ -646,9 +767,22 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   g_object_set (G_OBJECT (artist_renderer), "editable", FALSE, NULL);
   g_signal_handlers_block_by_func (track_listview, on_tracklist_row_activate, NULL);
 
+  if (! nautilus_burn_drive_lock (drive, _("Extracting audio from CD"), &reason)) {
+    g_warning ("Could not lock drive: %s", reason);
+    g_free (reason);
+  }
+  
+  cookie = sj_inhibit (g_get_application_name (),
+                       _("Extracting audio from CD"));
+
+  /* Save the genre */
+  save_genre (genre_entry);
+
   /* Start the extracting */
   extracting = TRUE;
-  pop_and_extract ();
+  gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &current);
+  find_next ();
+  pop_and_extract (&overwrite_mode);
 }
 
 /*
@@ -667,7 +801,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
  * string.
  */
 static char*
-sanitize_path (const char* str)
+sanitize_path (const char* str, const char* filesystem_type)
 {
   gchar *res = NULL;
   gchar *s;
@@ -679,6 +813,15 @@ sanitize_path (const char* str)
   s = g_strdup(str);
   /* Replace path seperators with a hyphen */
   g_strdelimit (s, "/", '-');
+  
+  /* filesystem specific sanitizing */
+  if (filesystem_type) {
+    if ((strcmp (filesystem_type, "vfat") == 0) ||
+        (strcmp (filesystem_type, "ntfs") == 0)) {
+      g_strdelimit (s, "\\:*?\"<>|", ' ');
+    }
+  }
+    
   if (strip_chars) {
     /* Replace separators with a hyphen */
     g_strdelimit (s, "\\:|", '-');
@@ -710,17 +853,31 @@ sanitize_path (const char* str)
  * %tA -- track artist (lowercase)
  * %ts -- track artist sortname
  * %tS -- track artist sortname (lowercase)
+ * %dn -- disc number (i.e 6)
+ * %dN -- disc number, zero padded (i.e 06)
  */
 char*
 filepath_parse_pattern (const char* pattern, const TrackDetails *track)
 {
   /* p is the pattern iterator, i is a general purpose iterator */
   const char *p;
-  char *tmp, *str;
+  char *tmp, *str, *base_path, *filesystem_type = NULL;
   GString *s;
+  GnomeVFSVolumeMonitor *monitor;
+  GnomeVFSVolume *volume;
 
   if (pattern == NULL || pattern[0] == 0)
     return g_strdup (" ");
+    
+  if ((base_path = gnome_vfs_get_local_path_from_uri (base_uri))) {
+    monitor = gnome_vfs_get_volume_monitor ();
+    
+    if ((volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, base_path))) {
+      filesystem_type = gnome_vfs_volume_get_filesystem_type (volume);
+      gnome_vfs_volume_unref (volume);
+    }
+    g_free (base_path);
+  }
 
   s = g_string_new (NULL);
 
@@ -755,27 +912,27 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->album->title);
+        string = sanitize_path (track->album->title, filesystem_type);
         break;
       case 'T':
         tmp = g_utf8_strdown (track->album->title, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free (tmp);
         break;
       case 'a':
-        string = sanitize_path (track->album->artist);
+        string = sanitize_path (track->album->artist, filesystem_type);
         break;
       case 'A':
         tmp = g_utf8_strdown (track->album->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free (tmp);
         break;
       case 's':
-        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist);
+        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, filesystem_type);
         break;
       case 'S':
         tmp = g_utf8_strdown (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       default:
@@ -794,27 +951,27 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->title);
+        string = sanitize_path (track->title, filesystem_type);
         break;
       case 'T':
         tmp = g_utf8_strdown (track->title, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 'a':
-        string = sanitize_path (track->artist);
+        string = sanitize_path (track->artist, filesystem_type);
         break;
       case 'A':
         tmp = g_utf8_strdown (track->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 's':
-        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist);
+        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist, filesystem_type);
         break;
       case 'S':
         tmp = g_utf8_strdown (track->artist_sortname ? track->album->artist_sortname : track->artist, -1);
-        string = sanitize_path (tmp);
+        string = sanitize_path (tmp, filesystem_type);
         g_free(tmp);
         break;
       case 'n':
@@ -835,7 +992,37 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
         go_next = FALSE;
       }
       break;
+    case 'd':
+      /*
+       * Disc and track tag
+       */
+      switch (*++p) {
+      case 'n':
+        /* Disc and track number */
+        if (track->album->disc_number > 0) {
+          string = g_strdup_printf ("Disc %d - %d", track->album->disc_number, track->number);
+        } else {
+          string = g_strdup_printf ("%d", track->number);
+        }
+        break;
+      case 'N':
+        /* Disc and track number, zero padded */
+        if (track->album->disc_number > 0) {
+          string = g_strdup_printf ("d%dt%02d", track->album->disc_number, track->number);
+        } else {
+          string = g_strdup_printf ("%02d", track->number);
+        }
+        break;
       default:
+        g_string_append (s, "%d");
+        p += 2;
+        
+        g_string_append_unichar (s, g_utf8_get_char (p));
+        p = g_utf8_next_char (p);
+        go_next = FALSE;
+      }
+      break;
+    default:
       /* append "%", and then the unicode character */
       g_string_append_c (s, '%');
       p += 1;
@@ -851,6 +1038,8 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
     if (go_next)
       ++p;
   }
+  
+  g_free (filesystem_type); 
 
   str = s->str;
   g_string_free (s, FALSE);
