@@ -112,37 +112,6 @@ static guint debug_flags = 0;
 #define SOURCE_GLADE "../data/sound-juicer.glade"
 #define INSTALLED_GLADE DATADIR"/sound-juicer/sound-juicer.glade"
 
-static void
-add_stock_icon (GtkIconFactory *sj_icon_factory,
-                const gchar *stock_id,
-                GtkIconSize  size,
-                const gchar *file_name)
-{
-  GtkIconSource *source;
-  GtkIconSet    *set;
-  GdkPixbuf     *pixbuf;
-
-  source = gtk_icon_source_new ();
-
-  gtk_icon_source_set_size (source, size);
-  gtk_icon_source_set_size_wildcarded (source, FALSE);
-
-  pixbuf = gdk_pixbuf_new_from_file (file_name, NULL);
-
-  gtk_icon_source_set_pixbuf (source, pixbuf);
-
-  g_object_unref (pixbuf);
-
-  set = gtk_icon_set_new ();
-
-  gtk_icon_set_add_source (set, source);
-  gtk_icon_source_free (source);
-
-  gtk_icon_factory_add (sj_icon_factory, stock_id, set);
-
-  gtk_icon_set_unref (set);
-}
-
 void
 sj_stock_init (void)
 {
@@ -151,8 +120,6 @@ sj_stock_init (void)
 
   static const GtkStockItem sj_stock_items[] =
   {
-    { SJ_STOCK_PLAYING, NULL, 0, 0, NULL },
-    { SJ_STOCK_RECORDING, NULL, 0, 0, NULL },
     { SJ_STOCK_EXTRACT, N_("E_xtract"), GDK_CONTROL_MASK, GDK_Return, NULL }
   };
 
@@ -161,8 +128,6 @@ sj_stock_init (void)
 
   sj_icon_factory = gtk_icon_factory_new ();
 
-  add_stock_icon (sj_icon_factory, SJ_STOCK_PLAYING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-play.png");
-  add_stock_icon (sj_icon_factory, SJ_STOCK_RECORDING, GTK_ICON_SIZE_MENU, PKGDATADIR"/sj-record.png");
   gtk_icon_factory_add (sj_icon_factory, SJ_STOCK_EXTRACT, gtk_icon_factory_lookup_default (GTK_STOCK_CDROM));
 
   gtk_icon_factory_add_default (sj_icon_factory);
@@ -346,10 +311,13 @@ static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
     g_object_set (G_OBJECT (cell), "stock-id", "", NULL);
     break;
   case STATE_PLAYING:
-    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_PLAYING, NULL);
+    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_PLAY, NULL);
+    break;
+  case STATE_PAUSED:
+    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_PAUSE, NULL);
     break;
   case STATE_EXTRACTING:
-    g_object_set (G_OBJECT (cell), "stock-id", SJ_STOCK_RECORDING, NULL);
+    g_object_set (G_OBJECT (cell), "stock-id", GTK_STOCK_MEDIA_RECORD, NULL);
     break;
   default:
     g_warning("Unhandled track state %d\n", state);
@@ -1399,29 +1367,46 @@ void on_title_edit_changed(GtkEditable *widget, gpointer user_data) {
 void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   GtkTreeIter iter;
   TrackDetails *track;
-  gchar *current_track_artist, *former_album_artist=NULL;
+  gchar *current_track_artist, *former_album_artist = NULL;
 
   g_return_if_fail (current_album != NULL);
+  
+  /* Unset the sortable artist field, as we can't change it automatically */
+  if (current_album->artist_sortname) {
+    g_free (current_album->artist_sortname);
+    current_album->artist_sortname = NULL;
+  }
+
   if (current_album->artist) {
-    former_album_artist=g_strdup(current_album->artist);
-    g_free (current_album->artist);
+    former_album_artist = current_album->artist;
   }
   current_album->artist = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 
-  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &iter)) return;
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &iter)) {
+    g_free (former_album_artist);
+    return;
+  }
   
   /* Set the artist field in each tree row */
   do {
-    gtk_tree_model_get(GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
     /* Change track artist if it matched album artist before the change */
     if ((strcasecmp (current_track_artist, former_album_artist) == 0) || (strcasecmp (current_track_artist, current_album->artist) == 0)) {
       gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
+      
       g_free (track->artist);
       track->artist = g_strdup (current_album->artist);
+      
+      if (track->artist_sortname) {
+        g_free (track->artist_sortname);
+        track->artist_sortname = NULL;
+      }
+      
       gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
-      gtk_tree_model_get(GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
     }
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
+  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (track_store), &iter));
+ 
+  g_free (former_album_artist);
 }
 
 void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
